#Noesis python importer - EA NHL games .rx2 models - version 1.55
from inc_noesis import *
import noesis

#rapi methods should only be used during handler callbacks
import rapi
#registerNoesisTypes is called by Noesis to allow the script to register formats.
#Do not implement this function in script files unless you want them to be dedicated format modules!
 
def registerNoesisTypes():
	handle = noesis.register("NHL Legacy (X360) models", ".rx2")
	noesis.setHandlerTypeCheck(handle, noepyCheckType)
	noesis.setHandlerLoadModel(handle, noepyLoadModel)
	#noesis.setHandlerLoadRGBA(handle, noetexLoadRGBA)
	
	noesis.logPopup()
	#print("The log can be useful for catching debug prints from preview loads.\nBut don't leave it on when you release your script, or it will probably annoy people.")
	return 1
#check if it's this type based on the data
def noepyCheckType(data):
    bs = NoeBitStream(data)
    Magic = bs.readBytes(7)
    if Magic != b'\x89\x52\x57\x34\x78\x62\x32':
        return 0
    return 1
 
texList = []
def noetexLoadRGBA(data, texList):
   del texList[:]
   bs = NoeBitStream(data, NOE_BIGENDIAN) 
   baseName = rapi.getExtensionlessName(rapi.getLocalFileName(rapi.getLastCheckedName()))
   TexInfo = []
   TexOff = []
   TexBufferSize = []
   NameInfo = []
   NameOff = []
   TexNames = []
   bs.seek(0x10, NOESEEK_ABS)
   version = bs.readBytes(3)
   bs.seek(13, NOESEEK_REL)   
   fileCount = bs.read(">i")
   bs.seek(12, NOESEEK_REL)
   fileTable = bs.read(">i")
   bs.seek(16, NOESEEK_REL)
   hdrSize = bs.read(">i")
   bs.seek(fileTable[0], NOESEEK_ABS)
   texCount = 0
   names = 0
   #check file types and file info		   			   		
   for i in range(0, fileCount[0]):
   		fileInfo = bs.read(">iiiii")
   		fileType = bs.readBytes(4)
   		if fileType == b'\x00\x02\x00\x03':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		TexOff.append([info[0]])
	   		TexBufferSize.append([info[2]])
	   		TexInfo.append([info[6] + 33])
	   		texCount +=1
	   			   	
   #Read all textures in texList		   			   		
   for i in range(0, texCount):
   		bs.seek(TexInfo[i][0], NOESEEK_ABS) #Seek to texture info
   		TexData = bs.read(">HBBBHiii") #Read WORD, BYTE, WORD, WORD, DWORD, DWORD, DWORD in Big endian
   		imgFmt = TexData[1]
   		imgHeight = (TexData[3] + 1) * 8
   		imgWidth = (TexData[4] + 1) & 0x1FFF

   		
   		TexNames.append(str(i + 1))
   		
   		bs.seek(TexOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to file start address
   		data = bs.readBytes(TexBufferSize[i][0]) #Read texture buffer
   		texFmt = 0
   		#DXT1
   		if imgFmt == 0x52:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 8)
   			texFmt = noesis.NOESISTEX_DXT1
   		#DXT3
   		elif imgFmt == 0x53:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			texFmt = noesis.NOESISTEX_DXT3
   		#DXT5
   		elif imgFmt == 0x54:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			texFmt = noesis.NOESISTEX_DXT5
   		#DXT5 packed normal map
   		elif imgFmt == 0x71:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			data = rapi.imageDecodeDXT(data, imgWidth, imgHeight, noesis.FOURCC_ATI2)
   			texFmt = noesis.NOESISTEX_RGBA32
   		#DXT1 packed normal map
   		elif imgFmt == 0x7C:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 8)
   			data = rapi.imageDecodeDXT(data, imgWidth, imgHeight, noesis.FOURCC_DXT1NORMAL)
   			texFmt = noesis.NOESISTEX_RGBA32	
   		#raw a8r8g8b8
   		elif imgFmt == 0x86:
   			if "name" in baseName:
   				data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "a8r8g8b8")
   				texFmt = noesis.NOESISTEX_RGBA32
   			else:
   				data = rapi.imageUntile360Raw(data, imgWidth, imgHeight, 4)
   				data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "a8r8g8b8")
   				texFmt = noesis.NOESISTEX_RGBA32
   		#raw b5g6r5
   		elif imgFmt == 0x44:
   			data = rapi.imageUntile360Raw(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 2)
   			data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "b5g6r5")
   			texFmt = noesis.NOESISTEX_RGBA32

   		#unknown, not handled
   		else:
   			print("WARNING: Unhandled image format " + repr(imgFmt) + " - " + repr(imgWidth) + "x" + repr(imgHeight) + " - " + repr(len(data)))
   			return None
   				   			    			  	   					 		
   		tex1 = NoeTexture(TexNames[i], imgWidth, imgHeight, data, texFmt)
   		texList.append(tex1)
   		

   return 1
#load the model

def noepyLoadModel(data, mdlList):
   GroupNames = []
   VInfo = []
   VOff = []
   VBufferSize = []
   FInfo = []   
   FOff = []
   FBufferSize = []
   MatInfo = []
   MatIndexInfo = []
   MatIndexTable = []
   TextureCompiler = []
   TexIndexInfo = []
   TexIndex = []
   TexI = []
   AmbientTexIndex = []
   NormalTexIndex = []
   SpecularTexIndex = []
   MeshInfo = []
   DescID = []
   UVpos = []
   UVtype = []
   UV2pos = []
   UV2type = []
   UV3pos = []
   UV3type = []
   UV4pos = []
   UV4type = []
   UV5pos = []
   UV5type = []
   COLpos = []
   COLtype = []
   BIpos = []
   BItype = []
   BWpos = []
   BWtype = []
   VBsize = []
   VertType = []
   MatNames = []
   LightMatNames = []
   MatNameIndex = []
   NameInfo = []
   MatList = []
   MaterialInfo = []
   MeshNames = []
   LightMapList = []
   isMap2 = []
   
   ctx = rapi.rpgCreateContext()
   bs = NoeBitStream(data, NOE_BIGENDIAN)
   bs.seek(0x20, NOESEEK_ABS)
   fileCount = bs.read(">i")
   bs.seek(12, NOESEEK_REL)
   fileTable = bs.read(">i")
   bs.seek(16, NOESEEK_REL)
   hdrSize = bs.read(">i")  
   bs.seek(fileTable[0], NOESEEK_ABS)
   meshCount = 0
   texCount = 0
   materialCount = 0
   ebCount = 0
   descriptorCount = 0

   
   for i in range(0, fileCount[0]):
	    fileInfo = bs.read(">iiiii")
	    fileType = bs.readBytes(4)
	   			    
	    if fileType == b'\x00\x02\x00\x05':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		VOff.append([info[0]])
	   		VBufferSize.append([info[2]])
	   		VInfo.append([info[6]])
	   		
	    elif fileType == b'\x00\x02\x00\x07':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		FOff.append([info[0]])
	   		FBufferSize.append([info[2]])
	   		FInfo.append([info[6]])	   		
	    elif fileType == b'\x00\x02\x00\x03':
	   		texCount +=1
	   		TexI.append(i)
	    elif fileType == b'\x00\x02\x00\x09':
	   		meshCount +=1 

	    elif fileType == b'\x00\x02\x00\x04':
	   		MeshInfo.append([fileInfo[0]])
	   		descriptorCount +=1
	   		DescID.append([i])
	   			   		
	    elif fileType == b'\x00\xEB\x00\x03':
	   		MatIndexInfo.append([fileInfo[0]])
	   		ebCount +=1
	   		
	    elif fileType == b'\x00\xEC\x00\x10':
	   		NameInfo.append([fileInfo[0]])
	    
	    elif fileType == b'\x00\xEF\x00\x00':
	   		TextureCompiler.append([fileInfo[0]])
	    
	    elif fileType == b'\x00\xEF\x00\x01':
	   		TexIndexInfo.append([fileInfo[0]])

	    elif fileType == b'\x00\xEF\x00\x05':
	   		MatInfo.append([fileInfo[0]])
	   		materialCount +=1				 			   		
   
#Read material names
   if ebCount != 0:
   		bs.seek(NameInfo[0][0], NOESEEK_ABS)
   		nameCount = bs.read(">HH")
   		bs.seek(4, NOESEEK_REL)
   		for i in range(0, nameCount[1]):
   			nameOff = bs.read(">i")
   			nameType = bs.readBytes(4)
   			if nameType == b'\x00\xEF\x00\x05':
   				MatNameIndex.append(bs.read(">i"))
   				pos = bs.tell()
   				bs.seek(nameOff[0], NOESEEK_ABS)
   				name = bs.readString()
   				MatNames.append(name)
   				LightMatNames.append(name + "LM")
   				bs.seek(pos, NOESEEK_ABS)
   			elif nameType == b'\x00\xEB\x00\x00':
   				GroupNameIndex = bs.read(">i")
   				pos = bs.tell()
   				bs.seek(nameOff[0], NOESEEK_ABS)
   				name = bs.readString()
   				print(GroupNameIndex[0])
   				print(name)
   				bs.seek(pos, NOESEEK_ABS)			
#Read material indices for meshes
   if materialCount != 0:
   		for i in range(0, ebCount):
   			bs.seek(MatIndexInfo[i][0], NOESEEK_ABS)
   			tables = bs.read(">i")
   			pos1 = bs.tell()
   			tabOff = bs.read(">iii")
   			for k in range(0, tables[0]):
   				bs.seek(pos1 + 4 + tabOff[k], NOESEEK_ABS)
   				contents = bs.read(">i")
   				for j in range(0, contents[0]):
   					idxOff = bs.read(">i")
   					pos2 = bs.tell()
   					bs.seek(MatIndexInfo[i][0] + idxOff[0] + 36, NOESEEK_ABS)
   					materialIndex = bs.read(">i")
   					MatIndexTable.append(materialIndex[0])
   					bs.seek(pos2, NOESEEK_ABS)

   				
   if texCount != 0:
   		sampleList = []
   		diffuseTexture = []
   		ambientTexture = []
   		normalTexture = []
   		coeffTexture = []
   		for i in range(0, materialCount):
   			bs.seek(TextureCompiler[i][0] + 8, NOESEEK_ABS)
   			sampleCount = bs.read("<i")
   			bs.seek(sampleCount[0]*4 + 4, NOESEEK_REL)
   			for j in range(0, sampleCount[0]):
   				sampleOff = bs.read("<i")
   				pos = bs.tell()
   				bs.seek(sampleOff[0], NOESEEK_ABS)
   				sampleList.append(bs.readString())
   				bs.seek(pos, NOESEEK_ABS)
   				
   			if 'diffuseTexture' in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('diffuseTexture')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				diffTex = bs.read(">i")
   				TexIndex.append(str((diffTex[0] - TexI[0]) // 2 + 1) + ".png")
   			elif 'map2' in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('map2')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				diffTex = bs.read(">i")
   				TexIndex.append(str((diffTex[0] - TexI[0]) // 2 + 1) + ".png")
   			elif 'map1' in sampleList and 'map2' not in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('map1')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				diffTex = bs.read(">i")
   				TexIndex.append(str((diffTex[0] - TexI[0]) // 2 + 1) + ".png")
   			else:
   				TexIndex.append(-1)

   			if 'ambientTexture' in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('ambientTexture')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				ambTex = bs.read(">i")
   				AmbientTexIndex.append(str((ambTex[0] - TexI[0]) // 2 + 1) + ".png")
   			else:
   				AmbientTexIndex.append(-1)
   			if 'normalMap' in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('normalMap')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				normTex = bs.read(">i")
   				NormalTexIndex.append(str((normTex[0] - TexI[0]) // 2 + 1) + ".png")
   			else:
   				NormalTexIndex.append(-1)
   			if 'specularMap' in sampleList:
   				bs.seek(TexIndexInfo[i][0] + 0x10 + sampleList.index('specularMap')*4, NOESEEK_ABS)
   				SOff = bs.read(">i")
   				bs.seek(SOff[0], NOESEEK_ABS)
   				specTex = bs.read(">i")
   				SpecularTexIndex.append(str((specTex[0] - TexI[0]) // 2 + 1) + ".png")
   			else:
   				SpecularTexIndex.append(-1)
   			if 'map2' in sampleList:
   				isMap2.append(1)
   			else:
   				isMap2.append(0)
   			del sampleList[:]
				
   if texCount != 0:		
   		for i in range(0, materialCount):
   			material = NoeMaterial(MatNames[i],"")
   			lightmapmaterial = NoeMaterial(LightMatNames[i],"")
   			if texCount != 0:
   				if TexIndex[i] != -1:
   					material.setTexture(TexIndex[i])
   				if NormalTexIndex[i] != -1:
   					material.setNormalTexture(NormalTexIndex[i])
   				if SpecularTexIndex[i] != -1:
   					material.setSpecularTexture(SpecularTexIndex[i])
   				if AmbientTexIndex != -1:
   					material.setOcclTexture(AmbientTexIndex[i])
   					lightmapmaterial.setTexture(AmbientTexIndex[i])
   				else:
   					material.setOcclTexture("-1")
   					lightmapmaterial.setTexture("-1")
   			MatList.append(material)
   			MatList.append(lightmapmaterial) 
		
   UVCount = 0
   UV2Count = 0
   UV3Count = 0
   UV4Count = 0
   UV5Count = 0
   COLCount = 0
   BICount = 0
   BWCount = 0

   for i in range(0, meshCount):
   		if descriptorCount < meshCount:
   			bs.seek(VInfo[i][0] + 40, NOESEEK_ABS)
   			partID = bs.read(">i")
   			bs.seek(MeshInfo[partID[0] - DescID[0][0]][0], NOESEEK_ABS)
   		else:
   			bs.seek(MeshInfo[i][0], NOESEEK_ABS)
   		info = bs.read(">iiBBBBBBBB")
   		VBsize.append(info[3])
   		for j in range(0, info[2]):
   			position = bs.read(">i")
   			type = bs.readBytes(8)
   			if type == b'\x00\x2A\x23\xB9\x00\x00\x00\x01':
   				VertType.append(1)   			
   			elif type == b'\x00\x1A\x23\x60\x00\x00\x00\x01':
   				VertType.append(2)  			
   			elif type == b'\x00\x1A\x23\xA6\x00\x00\x00\x01':
   				VertType.append(3)
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(1)
   				UVCount += 1
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x01\x07':
   				UV2pos.append(position[0])
   				UV2type.append(1)
   				UV2Count += 1
   			
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x02\x08':
   				UV3pos.append(position[0])
   				UV3type.append(1)
   				UV3Count += 1
   			
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x03\x09':
   				UV4pos.append(position[0])
   				UV4type.append(1)
   				UV4Count += 1
   			
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x04\x0A':
   				UV5pos.append(position[0])
   				UV5type.append(1)
   				UV5Count += 1
   			
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x01\x07':
   				UV2pos.append(position[0])
   				UV2type.append(2)
   				UV2Count += 1
   			
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x02\x08':
   				UV3pos.append(position[0])
   				UV3type.append(2)
   				UV3Count += 1
   			
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x03\x09':
   				UV4pos.append(position[0])
   				UV4type.append(2)
   				UV4Count += 1	
   			
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x04\x0A':
   				UV5pos.append(position[0])
   				UV5type.append(2)
   				UV5Count += 1	
   			
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(2)
   				UVCount += 1
   			
   			elif type == b'\x00\x2C\x20\x59\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(3)
   				UVCount += 1   			

   			elif type == b'\x00\x2C\x21\x59\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(4)
   				UVCount += 1
   				
   			elif type == b'\x00\x18\x28\x86\x00\x0A\x00\x04':
   				COLpos.append(position[0])
   				COLtype.append(1)
   				COLCount += 1
   			elif type == b'\x00\x1A\x22\x5A\x00\x02\x00\x0E':
   				BIpos.append(position[0])
   				BItype.append(1)
   				BICount += 1

   			elif type == b'\x00\x1A\x20\x86\x00\x01\x00\x0F':
   				BWpos.append(position[0])
   				BWtype.append(1)
   				BWCount += 1
   				 			
   		if UVCount == i:
   			UVtype.append(0)
   			UVpos.append(0)
   			UVCount += 1
   		
   		if UV2Count == i:
   			UV2type.append(0)
   			UV2pos.append(0)
   			UV2Count += 1
   		
   		if UV3Count == i:
   			UV3type.append(0)
   			UV3pos.append(8)
   			UV3Count += 1
   		
   		if UV4Count == i:
   			UV4type.append(0)
   			UV4pos.append(0)
   			UV4Count += 1
   		
   		if UV5Count == i:
   			UV5type.append(0)
   			UV5pos.append(0)
   			UV5Count += 1
   		
   		if COLCount == i:
   			COLtype.append(0)
   			COLpos.append(8)
   			COLCount += 1
   		
   		if BICount == i:
   			BItype.append(0)
   			BIpos.append(8)
   			BICount += 1
   		
   		if BWCount == i:
   			BWtype.append(0)
   			BWpos.append(8)
   			BWCount += 1
   					  				
 #Construct Models		  			   
   for i in range(0, meshCount):
   		rapi.rpgClearBufferBinds()
   		B = 0
   		bs.seek(VInfo[i][0] + 32, NOESEEK_ABS) #Seek to vertices info
   		VData = bs.read(">HHi")
   		VBsize = VData[0]
   		VCount = VData[2]
   		bs.seek(FOff[i][0] + hdrSize[0], NOESEEK_ABS)
   		TriStrip = 0
   		#check if the faces are triangleStrip format
   		checkBuffer = FBufferSize[i][0] // 2
   		for j in range(0, checkBuffer):
   			StripCheck = bs.read(">H")
   			if StripCheck[0] == 0xFFFF:
   				TriStrip = 1
   		bs.seek(FInfo[i][0], NOESEEK_ABS) #Seek to faces info
   		bs.seek(32, NOESEEK_REL)
   		FCount = bs.read(">HH")
   		#sometimes triangle count isn't divisible by 3. Subtract B from triangle count
   		if FCount [1] % 3 != 0:
   			B += 2  
   		bs.seek(VOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to Vertices Start
   		VertBuff = bs.readBytes(VBufferSize[i][0])
   		rapi.rpgSetName("mesh_" + str(i))
   		rapi.rpgSetOption(noesis.RPGOPT_BIGENDIAN, 1)
   		
   		if VertType[i] == 1:
   			rapi.rpgBindPositionBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 0)
   		
   		elif VertType[i] == 2:
   			rapi.rpgBindPositionBufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, 0)   		
   		
   		elif VertType[i] == 3:
   			rapi.rpgBindPositionBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 0)
   		
   		if UVtype[i] == 1:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UVpos[i])
   		
   		elif UVtype[i] == 2:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, UVpos[i])
   		
   		elif UVtype[i] == 3:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, UVpos[i])   			 			
   		
   		elif UVtype[i] == 4:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_SHORT, VBsize, UVpos[i])
   		
   		
   		if UV2type[i] == 1:
   			if materialCount != 0:
   				if isMap2[(MatIndexTable[i] - MatNameIndex[0][0]) // 3] == 0:
   					rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UV2pos[i])
   				else:
   					rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UV3pos[i])
   			else:
   				rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UV2pos[i])
   			
   		elif UV2type[i] == 2:
   			if materialCount != 0:
   				if isMap2[(MatIndexTable[i] - MatNameIndex[0][0]) // 3] == 0:
   					rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, UV2pos[i])
   				else:
   					rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, UV3pos[i])	
   			else:
   				rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, UV2pos[i])
   		
   		
   		if UV3type[i] == 1:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 3, 3, UV3pos[i])
   		
   		elif UV3type[i] == 2:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, 3, 3, UV3pos[i])
   		
   		elif UV3type[i] == 3:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, 3, 3, UV3pos[i])   			 			
   		
   		elif UV3type[i] == 4:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_SHORT, VBsize, 3, 3, UV3pos[i])
   		
   		if UV4type[i] == 1:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 4, 4, UV4pos[i])
   		
   		elif UV4type[i] == 2:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, 4, 4, UV4pos[i])
   		
   		elif UV4type[i] == 3:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, 4, 4, UV4pos[i])   			 			
   		
   		elif UV4type[i] == 4:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_SHORT, VBsize, 4, 4, UV4pos[i])
   		
   		if UV5type[i] == 1:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 5, 5, UV5pos[i])
   		
   		elif UV5type[i] == 2:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, 5, 5, UV5pos[i])
   		
   		elif UV5type[i] == 3:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, 5, 5, UV5pos[i])   			 			
   		
   		elif UV5type[i] == 4:
   			rapi.rpgBindUVXBufferOfs(VertBuff, noesis.RPGEODATA_SHORT, VBsize, 5, 5, UV5pos[i])
   		
   		if COLtype[i] == 1:
   			rapi.rpgBindColorBufferOfs(VertBuff, noesis.RPGEODATA_UBYTE, VBsize, COLpos[i], 4)
   		
   		if BItype[i] == 1:
   			rapi.rpgBindBoneIndexBufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, BIpos[i], 2)
   		if BWtype[i] == 1:
   			rapi.rpgBindBoneWeightBufferOfs(VertBuff, noesis.RPGEODATA_UBYTE, VBsize, BWpos[i], 4)
   			   			  				
   		bs.seek(FOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to Faces Start
   		FaceBuff = bs.readBytes(FBufferSize[i][0])
   		
   		if texCount != 0:
   			rapi.rpgSetMaterial(MatNames[(MatIndexTable[i] - MatNameIndex[0][0]) // 3])
   			if AmbientTexIndex[(MatIndexTable[i] - MatNameIndex[0][0]) // 3] != -1:
   				rapi.rpgSetLightmap(LightMatNames[(MatIndexTable[i] - MatNameIndex[0][0]) // 3])
	
   		if TriStrip == 1:
   			rapi.rpgCommitTriangles(FaceBuff, noesis.RPGEODATA_USHORT, (FCount[1]), noesis.RPGEO_TRIANGLE_STRIP, 1)
   		else:
   			rapi.rpgCommitTriangles(FaceBuff, noesis.RPGEODATA_USHORT, FCount[1] - B, noesis.RPGEO_TRIANGLE, 1)
   if texCount != 0:
   		noetexLoadRGBA(data, texList)
   if meshCount != 0:		
   		mdl = rapi.rpgConstructModel()
   if materialCount != 0:
   		mdl.setModelMaterials(NoeModelMaterials(texList, MatList))
   mdlList.append(mdl)
   rapi.rpgClearBufferBinds()
   rapi.rpgReset() 
   return 1
   